
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from django.utils.safestring import mark_safe

from shop.offers.models import ProductPriceOffer


class ProductPriceOfferAdmin(admin.ModelAdmin):

    list_display = [
        'product', 'name', 'mobile', 'email', 'user', 'date_created', 'status',
        'product_link'
    ]
    list_editable = ['status']
    search_fields = ['name', 'mobile', 'email']
    list_filter = ['status']

    def product_link(self, item):
        return mark_safe('<a href="%s" target="_blank">%s</a>' % (
            item.product.get_absolute_url(), _('View product')))

    product_link.short_description = _('Product')


admin.site.register(ProductPriceOffer, ProductPriceOfferAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2017-07-18 15:17
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('orders', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='order',
            name='delivery_method',
            field=models.PositiveIntegerField(choices=[(1, 'Self delivery'), (2, 'Nova Poshta'), (3, 'In Time'), (4, 'Gunsel'), (5, 'Avtoluks'), (6, 'Other')], default=None, verbose_name='Delivery method'),
        ),
        migrations.AlterField(
            model_name='order',
            name='email',
            field=models.EmailField(blank=True, max_length=255, null=True, verbose_name='Email'),
        ),
    ]




from django.conf.urls import url, include

from shop.flags import views


urlpatterns = [

    url(r'^api/', include('shop.flags.api.urls', namespace='api')),

    url(r'^flag/(?P<flag_pk>\d+)/$', views.get_products_by_flag,
        name='products')

]




from django.db import models
from django.utils import six
from django.utils.translation import ugettext_lazy as _

from shop.attributes.constants import (
    ATTR_TYPE_BOOL, ATTR_TYPE_INT, ATTR_TYPE_TEXT, ATTR_TYPE_SELECT, ATTR_TYPES
)


ATTR_VALUE_TEXT = models.TextField(_('Text'), blank=True, null=True)
ATTR_VALUE_INT = models.IntegerField(_('Integer'), blank=True, null=True)
ATTR_VALUE_BOOL = models.NullBooleanField(_('Boolean'), blank=True)
ATTR_VALUE_OPTION = models.ForeignKey(
    'attributes.AttributeOption', blank=True, null=True,
    verbose_name=_("Value option"))

VALUE_FIELDS = {
    ATTR_TYPE_TEXT: ATTR_VALUE_TEXT,
    ATTR_TYPE_INT: ATTR_VALUE_INT,
    ATTR_TYPE_BOOL: ATTR_VALUE_BOOL,
    ATTR_TYPE_SELECT: ATTR_VALUE_OPTION,
}


class AttributeOptionGroup(models.Model):

    name = models.CharField(_('Name'), max_length=128)

    def __unicode__(self):
        return self.name

    class Meta:
        verbose_name = _('Attribute option group')
        verbose_name_plural = _('Attribute option groups')


class AttributeOption(models.Model):

    group = models.ForeignKey(
        'attributes.AttributeOptionGroup', related_name='options',
        verbose_name=_("Group"))

    option = models.CharField(_('Option'), max_length=255)

    def __unicode__(self):
        return self.option

    class Meta:
        unique_together = ('group', 'option')
        verbose_name = _('Attribute option')
        verbose_name_plural = _('Attribute options')


class ProductAttribute(models.Model):

    category = models.ForeignKey(
        'products.ProductCategory', related_name='attributes', blank=True,
        null=True, verbose_name=_("Product category"))

    name = models.CharField(_('Name'), max_length=128)

    slug = models.CharField(_('Code'), max_length=255, db_index=True,
                            blank=True, null=False)

    type = models.PositiveSmallIntegerField(
        choices=ATTR_TYPES, default=ATTR_TYPE_TEXT,
        null=False, verbose_name=_("Type"))

    option_group = models.OneToOneField(
        'attributes.AttributeOptionGroup', blank=True, null=True,
        related_name='attribute', verbose_name=_("Option Group"),
        help_text=_('Select an option group if using type "Option"'))

    required = models.BooleanField(_('Required'), default=False)

    is_filter = models.BooleanField(_('Is filter'), default=False)

    @property
    def has_options(self):
        return self.type == ATTR_TYPE_SELECT

    @property
    def full_slug(self):
        return 'attr_%s' % self.slug

    def __unicode__(self):
        return self.name

    def save_value(self, product, value):

        value_obj, c = product.attribute_values.get_or_create(attribute=self)

        if value is None or value == '':
            value_obj.delete()
            return

        if value != value_obj.value:
            value_obj.value = value
            value_obj.save()

    class Meta:
        ordering = ['slug']
        verbose_name = _('Product attribute')
        verbose_name_plural = _('Product attributes')


class ProductAttributeValue(models.Model):

    attribute = models.ForeignKey(
        'attributes.ProductAttribute', verbose_name=_("Attribute"),
        related_name='values')

    product = models.ForeignKey(
        'products.Product', related_name='attribute_values',
        verbose_name=_("Product"))

    value_text = ATTR_VALUE_TEXT
    value_int = ATTR_VALUE_INT
    value_bool = ATTR_VALUE_BOOL
    value_option = ATTR_VALUE_OPTION

    @staticmethod
    def get_value_field(field_type):
        return VALUE_FIELDS[field_type]

    @property
    def value_field(self):
        return self.get_value_field(self.attribute.type)

    def get_value(self):
        return getattr(self, self.value_field.name)

    def set_value(self, new_value):

        if self.attribute.has_options and \
                isinstance(new_value, six.string_types):

            new_value = self.attribute.option_group.options.get(
                option=new_value)

        setattr(self, self.value_field.name, new_value)

    value = property(get_value, set_value)

    def as_text(self):
        value = self.get_value()

        if self.attribute.type == ATTR_TYPE_BOOL:
            if value is None:
                return ''
            return _('Yes') if value else _('No')

        return unicode('' if value is None else value)

    def as_html(self):
        return self.as_text()

    def __unicode__(self):
        return u"%s: %s" % (self.attribute.name, self.as_text())

    class Meta:
        ordering = ('attribute__name', )
        unique_together = ('attribute', 'product')
        verbose_name = _('Product attribute value')
        verbose_name_plural = _('Product attribute values')


class ProductAttrsContainer(object):

    def __setstate__(self, state):
        self.__dict__ = state
        self.initialised = False

    def __init__(self, product):
        self.product = product
        self.initialised = False
        self._attrs = None

    def __getattr__(self, name):
        if not name.startswith('_') and not self.initialised:
            values = self.get_values().select_related('attribute')
            for v in values:
                setattr(self, v.attribute.slug, v.value)
            self.initialised = True
            return getattr(self, name)
        return None

    def get_values(self):
        return self.product.attribute_values.all()

    def get_value_by_attribute(self, attribute):
        return self.get_values().get(attribute=attribute)

    def all(self):
        if not self.product.pk:
            return []

        if not self._attrs:
            self._attrs = self.product.category.get_attributes()

        return self._attrs

    def get_attribute_by_code(self, code):
        return self.all().get(code=code)

    def __iter__(self):
        return iter(self.get_values())

    def save(self):
        for attribute in self.all():
            if hasattr(self, attribute.slug):
                value = getattr(self, attribute.slug)
                attribute.save_value(self.product, value)


class ProductCategoryAttrsMixin(object):

    def get_attributes(self):
        return ProductAttribute.objects.filter(
            models.Q(category=self) | models.Q(category__isnull=True))


class ProductAttrsMixin(object):

    attributes = models.ManyToManyField(
        'attributes.ProductAttribute', through='ProductAttributeValue',
        verbose_name=_("Attributes"))

    def __init__(self, *args, **kwargs):

        super(ProductAttrsMixin, self).__init__(*args, **kwargs)

        self.attr = ProductAttrsContainer(self)

    def save(self, *args, **kwargs):

        is_new = bool(not self.pk)

        super(ProductAttrsMixin, self).save(*args, **kwargs)

        if not is_new:
            self.attr.save()




from django.apps import apps
from django import forms


class CheckoutForm(forms.ModelForm):

    class Meta:
        model = apps.get_model('orders', 'Order')
        fields = [
            'name', 'surname', 'payment_method', 'delivery_method',
            'post_office', 'mobile', 'email', 'comment'
        ]




from django.apps import apps
from django.db.models import F


def get_endangered_products():
    Product = apps.get_model('products', 'Product')
    return Product.objects.filter(min_qty__gt=F('qty'))




from django import template


register = template.Library()


@register.assignment_tag
def get_product_flags():
    from shop.flags.models import ProductFlag
    return ProductFlag.objects.all().prefetch_related('products')




from django.conf.urls import url

from shop.offers import views


urlpatterns = [

    url(r'^modal/(?P<product_pk>\d+)/$', views.get_price_offer_modal,
        name='modal'),

    url(r'^send/(?P<product_pk>\d+)/$', views.send_price_offer, name='send'),

]




from django.apps import apps
from django import forms
from django.utils.translation import ugettext_lazy as _

from suit.sortables import SortableTabularInline
from multiupload.fields import MultiFileField


class ProductForm(forms.ModelForm):

    images = MultiFileField(
        label=_('Images'), max_num=100, min_num=1, required=False)

    class Meta:
        model = apps.get_model('products', 'Product')
        fields = '__all__'


class ProductImageInline(SortableTabularInline):
    fields = ('preview', )
    readonly_fields = ['preview']
    model = apps.get_model('products', 'ProductImage')
    extra = 0
    max_num = 0





from modeltranslation.translator import translator, TranslationOptions

from shop.attributes.models import ProductAttribute, AttributeOption


class ProductAttributeTranslationOptions(TranslationOptions):

    fields = ('name', )


class AttributeOptionTranslationOptions(TranslationOptions):

    fields = ('option', )


translator.register(ProductAttribute, ProductAttributeTranslationOptions)
translator.register(AttributeOption, AttributeOptionTranslationOptions)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2017-02-22 18:08
from __future__ import unicode_literals

from django.db import migrations, models

from shop.currencies.settings import CURRENCIES


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ExchangeRate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('currency', models.PositiveIntegerField(choices=CURRENCIES, unique=True, verbose_name='Currency')),
                ('value', models.FloatField(default=0, verbose_name='Value')),
            ],
            options={
                'verbose_name': 'Exchange rate',
                'verbose_name_plural': 'Exchange rate',
            },
        ),
    ]




from django.shortcuts import render

from shop.flags.models import ProductFlag


def get_flags(request):

    flags = ProductFlag.objects.all().prefetch_related('products')

    return render(request, 'flags/api/flags.html', {'flags': flags})




from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse


class ProductFlag(models.Model):

    title = models.CharField(_('Title'), max_length=255, blank=True)

    def __unicode__(self):
        return self.title

    def get_random_products(self, count=6):
        return self.products.visible().order_by('?')[:count]

    def get_absolute_url(self):
        return reverse('products:by-flag', kwargs={'flag_pk': self.pk})

    class Meta:
        verbose_name = _('Product flag')
        verbose_name_plural = _('Product flags')




from django.utils.translation import ugettext_lazy as _


AVAILABILITY_IN_STOCK = 0
AVAILABILITY_ORDER = 1
AVAILABILITY_NOT_AVAILABLE = 2

AVAILABILITY_CHOICES = (
    (AVAILABILITY_IN_STOCK, _('In stock')),
    (AVAILABILITY_NOT_AVAILABLE, _('Not available')),
    (AVAILABILITY_ORDER, _('Should be ordered')),
)




from django import template

from shop.currencies.settings import CURRENCY_SESSION_KEY
from shop.currencies.forms import CurrencyForm


register = template.Library()


@register.assignment_tag(takes_context=True)
def get_currency_form(context):
    currency = context.request.session.get(CURRENCY_SESSION_KEY)
    return CurrencyForm(initial={'currency': currency})




from django.apps import apps
from django.conf import settings
from django import forms
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from shop.lib import get_show_on_site_link


class OrderProductInline(admin.TabularInline):

    model = apps.get_model('orders', 'OrderProduct')
    extra = 0


class OrderProductForm(forms.ModelForm):

    def clean_title(self):
        data = self.cleaned_data

        title = data.get('title')
        parent = data.get('parent')

        if parent and not title:
            title = parent.title

        if not parent and not title:
            raise forms.ValidationError(_('Select product or enter title'))

        return title

    def clean_price(self):
        data = self.cleaned_data

        price = data.get('price')
        parent = data.get('parent')

        if parent and not price:
            price = parent.price

        if not parent and not price:
            raise forms.ValidationError(_('Select product or enter price'))

        return price

    class Meta:
        model = apps.get_model('orders', 'OrderProduct')
        fields = '__all__'


class OrderAdmin(admin.ModelAdmin):

    inlines = [OrderProductInline]

    list_display = [
        'printable_name', 'name', 'mobile', 'post_office', 'email',
        'date_created', 'product_count', 'printable_default_total', 'status',
        'get_preview'
    ]

    search_fields = ['id', 'name', 'mobile', 'post_office', 'email']

    list_editable = ['status']

    def get_preview(self, item):

        try:
            return item.products.first().parent.preview
        except Exception:
            return '----'

    get_preview.short_description = _('Preview')


class OrderProductAdmin(admin.ModelAdmin):

    form = OrderProductForm

    list_display = [
        'product_title', 'price', 'qty', 'order_name', 'get_show_link',
        'get_preview'
    ]

    search_fields = ['title', 'price', 'parent__code', 'parent__title']

    list_filter = ['order']

    def get_show_link(self, item):

        if item.parent:
            return get_show_on_site_link(item.parent.get_absolute_url())

        return '----'

    get_show_link.short_description = _('Show on site')

    def get_preview(self, item):

        if item.parent:
            return item.parent.preview

        return '----'

    get_preview.short_description = _('Preview')


if 'shop.orders' in settings.INSTALLED_APPS:

    from shop.orders.models import Order, OrderProduct

    admin.site.register(Order, OrderAdmin)
    admin.site.register(OrderProduct, OrderProductAdmin)




from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from django.db import models
from django.core.validators import MinValueValidator

from shop.currencies.lib import format_printable_price
from shop.orders.settings import (
    ORDER_DELIVERY_METHODS, ORDER_PAYMENT_METHODS, ORDER_STATUSES,
    DEFAULT_STATUS, DEFAULT_DELIVERY_METHOD, DEFAULT_PAYMENT_METHOD
)


class AbstractOrder(models.Model):

    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, related_name='order',
        verbose_name=_('Customer'), null=True, blank=True)

    payment_method = models.PositiveIntegerField(
        _('Payment method'), null=False,
        blank=DEFAULT_PAYMENT_METHOD is not None,
        choices=ORDER_PAYMENT_METHODS,
        default=DEFAULT_PAYMENT_METHOD
    )

    delivery_method = models.PositiveIntegerField(
        _('Delivery method'), null=False,
        blank=DEFAULT_DELIVERY_METHOD is not None,
        choices=ORDER_DELIVERY_METHODS,
        default=DEFAULT_DELIVERY_METHOD
    )

    status = models.PositiveIntegerField(
        _('Status'), null=False, choices=ORDER_STATUSES,
        default=DEFAULT_STATUS)

    name = models.CharField(_("Name"), max_length=255, blank=False, null=False)

    surname = models.CharField(
        _("Surname"), max_length=255, blank=False, null=False)

    post_office = models.CharField(
        _("Address and number of post office"),
        max_length=255, blank=True, null=True)

    mobile = models.CharField(
        _("Mobile phone"), max_length=255, blank=False, null=False)

    email = models.EmailField(
        _("Email"), max_length=255, blank=True, null=True)

    date_created = models.DateTimeField(
        _('Date created'), auto_now_add=True, editable=False)

    comment = models.TextField(
        _('Comment'), max_length=1000, blank=True, default='')

    @property
    def printable_name(self):
        return _('Order #%s') % self.id

    printable_name.fget.short_description = _('Order')

    def __unicode__(self):
        return self.printable_name

    @property
    def product_count(self):
        return sum([item.qty for item in self.products.all()])

    product_count.fget.short_description = _('Product count')

    @property
    def default_total(self):
        return sum([item.default_subtotal for item in self.products.all()])

    @property
    def printable_default_total(self):
        return format_printable_price(self.default_total)

    printable_default_total.fget.short_description = _('Total')

    class Meta:
        abstract = True
        verbose_name = _('Order')
        verbose_name_plural = _('Orders')


class AbstractOrderProduct(models.Model):

    order = models.ForeignKey(
        'orders.Order', related_name='products', verbose_name=_('Order'))

    parent = models.ForeignKey(
        'products.Product', verbose_name=_('Product'),
        on_delete=models.SET_NULL, blank=True, null=True)

    title = models.CharField(_("Product title"), max_length=255, blank=True)

    price = models.FloatField(_('Price (uah)'), blank=False, null=False)

    qty = models.PositiveIntegerField(
        _('Quantity'), default=1, validators=[MinValueValidator(1)])

    @property
    def order_name(self):
        return self.order.printable_name

    order_name.fget.short_description = _('Order')

    @property
    def default_subtotal(self):
        return self.price * self.qty

    @property
    def printable_default_price(self):
        return format_printable_price(self.price)

    @property
    def printable_default_subtotal(self):
        return format_printable_price(self.default_subtotal)

    @property
    def code(self):
        if self.parent:
            return self.parent.code

        return ''

    @property
    def product_title(self):
        if self.parent:
            return self.parent.title

        return self.title

    product_title.fget.short_description = _('Title')

    def __unicode__(self):
        return self.product_title

    class Meta:
        abstract = True
        verbose_name = _('Order product')
        verbose_name_plural = _('Order products')




from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from modeltranslation.admin import TranslationAdmin

from shop.attributes.admin.forms import (
    ProductAttributeForm, AttributeOptionInline)
from shop.attributes.models import AttributeOptionGroup, ProductAttribute


class ProductAttributeAdmin(TranslationAdmin):

    form = ProductAttributeForm

    list_display = (
        'name', 'category', 'slug', 'get_type_display', 'required',
        'is_filter',
    )
    search_fields = ('name', 'slug', )
    list_filter = ('category', 'type', 'required', )


class AttributeOptionGroupAdmin(admin.ModelAdmin):

    inlines = [AttributeOptionInline]

    list_display = ('name', 'attribute', 'category', )
    search_fields = ('name', 'attribute__name', )
    list_filter = ('attribute__category', )

    def category(self, item):
        return item.attribute.category
    category.short_description = _('Category')


class ProductAdminMixin(object):

    def render_change_form(self, request, context, **kwargs):

        form = context['adminform'].form

        fieldsets = self.fieldsets or [(None, {'fields': form.fields.keys()})]

        context['adminform'] = admin.helpers.AdminForm(
            form, fieldsets, self.prepopulated_fields, model_admin=self)

        return super(ProductAdminMixin, self).render_change_form(
            request, context, **kwargs)


admin.site.register(ProductAttribute, ProductAttributeAdmin)
admin.site.register(AttributeOptionGroup, AttributeOptionGroupAdmin)



from django.conf import settings

from modeltranslation.translator import TranslationOptions, translator


class ProductTranslationOptions(TranslationOptions):

    fields = ('title', 'description', )


class ProductCategoryTranslationOptions(TranslationOptions):

    fields = ('name', )


if 'shop.products' in settings.INSTALLED_APPS:

    from shop.products.models import Product, ProductCategory

    translator.register(Product, ProductTranslationOptions)
    translator.register(ProductCategory, ProductCategoryTranslationOptions)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2017-03-22 14:42
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('products', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='AttributeOption',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('option', models.CharField(max_length=255, verbose_name='Option')),
                ('option_uk', models.CharField(max_length=255, null=True, verbose_name='Option')),
            ],
            options={
                'verbose_name': 'Attribute option',
                'verbose_name_plural': 'Attribute options',
            },
        ),
        migrations.CreateModel(
            name='AttributeOptionGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=128, verbose_name='Name')),
            ],
            options={
                'verbose_name': 'Attribute option group',
                'verbose_name_plural': 'Attribute option groups',
            },
        ),
        migrations.CreateModel(
            name='ProductAttribute',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=128, verbose_name='Name')),
                ('name_uk', models.CharField(max_length=128, null=True, verbose_name='Name')),
                ('slug', models.CharField(blank=True, db_index=True, max_length=255, verbose_name='Code')),
                ('type', models.PositiveSmallIntegerField(choices=[(1, 'Text'), (2, 'Integer'), (3, 'True / False'), (5, 'Option')], default=1, verbose_name='Type')),
                ('required', models.BooleanField(default=False, verbose_name='Required')),
                ('is_filter', models.BooleanField(default=False, verbose_name='Is filter')),
                ('category', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='attributes', to='products.ProductCategory', verbose_name='Product category')),
                ('option_group', models.OneToOneField(blank=True, help_text='Select an option group if using type "Option"', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='attribute', to='attributes.AttributeOptionGroup', verbose_name='Option Group')),
            ],
            options={
                'ordering': ['slug'],
                'verbose_name': 'Product attribute',
                'verbose_name_plural': 'Product attributes',
            },
        ),
        migrations.CreateModel(
            name='ProductAttributeValue',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value_text', models.TextField(blank=True, null=True, verbose_name='Text')),
                ('value_int', models.IntegerField(blank=True, null=True, verbose_name='Integer')),
                ('value_bool', models.NullBooleanField(verbose_name='Boolean')),
                ('attribute', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='values', to='attributes.ProductAttribute', verbose_name='Attribute')),
                ('product', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='attribute_values', to='products.Product', verbose_name='Product')),
                ('value_option', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='attributes.AttributeOption', verbose_name='Value option')),
            ],
            options={
                'ordering': ('attribute__name',),
                'verbose_name': 'Product attribute value',
                'verbose_name_plural': 'Product attribute values',
            },
        ),
        migrations.AddField(
            model_name='attributeoption',
            name='group',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='options', to='attributes.AttributeOptionGroup', verbose_name='Group'),
        ),
        migrations.AlterUniqueTogether(
            name='productattributevalue',
            unique_together=set([('attribute', 'product')]),
        ),
        migrations.AlterUniqueTogether(
            name='attributeoption',
            unique_together=set([('group', 'option')]),
        ),
    ]




from django import forms

from captcha.fields import ReCaptchaField

from shop.offers.models import ProductPriceOffer


class ProductPriceOfferForm(forms.ModelForm):

    captcha = ReCaptchaField()

    class Meta:
        model = ProductPriceOffer
        fields = ('name', 'mobile', 'email', 'text', )




from django.contrib import admin

from modeltranslation.admin import TranslationAdmin

from shop.flags.models import ProductFlag


class ProductFlagAdmin(TranslationAdmin):
    pass


admin.site.register(ProductFlag, ProductFlagAdmin)




from django.apps import apps

__all__ = ['Comparison']


class ComparisonItem(object):

    def __init__(self, product):
        self.product = product

    def __repr__(self):
        return self.product.title

    def to_dict(self):
        return {
            'product_pk': self.product.pk,
            'category_pk': self.product.category_id
        }


class Comparison(object):

    def __init__(self, session, session_key='comparison'):

        self._items_dict = {}

        self.session = session
        self.session_key = session_key

        if self.session_key in self.session:

            representation = self.session[self.session_key]

            ids = representation.keys()

            Product = apps.get_model('products', 'Product')

            for product in Product.objects.filter(pk__in=ids)\
                    .select_related('category'):
                self._items_dict[product.pk] = ComparisonItem(product)

    def __contains__(self, product):
        return product in self.products

    def get_product(self, pk):
        return self._items_dict.get(pk)

    def update_session(self):
        self.session[self.session_key] = self.serialize()
        self.session.modified = True

    def add(self, product):

        if product not in self.products:
            self._items_dict[product.pk] = ComparisonItem(product)
            self.update_session()

    def remove(self, product):
        if product in self.products:
            del self._items_dict[product.pk]
            self.update_session()

    def clear(self):
        self._items_dict = {}
        self.update_session()

    @property
    def items(self):
        return self._items_dict.values()

    @property
    def count(self):
        return len(self._items_dict)


    @property
    def is_empty(self):
        return self.count == 0

    @property
    def products(self):
        return [item.product for item in self.items]

    def get_products(self, category):
        products = []
        for item in self.items:
            if item.product.category.pk == category.pk:
                products.append(item.product)
        return products

    @property
    def categories(self):
        items = []
        for item in self.items:
            if not item.product.category in items:
                items.append(item.product.category)
        return items

    def serialize(self):
        items = {}
        for item in self.items:
            product_id = str(item.product.pk)
            items[product_id] = item.to_dict()
        return items




from django.apps import apps
from django.conf import settings
from django.core.mail import mail_managers, send_mail
from django.utils.translation import ugettext_lazy as _
from django.template.loader import render_to_string


def send_new_order_notifications(order):

    from django.contrib.sites.models import Site

    subject = _('New order #%s') % order.id

    context = {'order': order, 'site': Site.objects.get_current()}

    email_template_name = 'orders/email/new_order_notification_for_admins.html'

    html = render_to_string(email_template_name, context)

    mail_managers(subject=subject, message='', html_message=html)

    if apps.is_installed('turbosms'):

        from turbosms.lib import send_sms_from_template

        sms_template_name = 'orders/sms/new_order_notification_for_admins.txt'

        send_sms_from_template(sms_template_name, context)

    if order.email:

        email_template_name = (
            'orders/email/new_order_notification_for_customer.html')

        html = render_to_string(email_template_name, context)

        send_mail(subject, '', settings.EMAIL_HOST_USER, [order.email],
                  html_message=html)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2017-03-22 14:44
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ProductFlag',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(blank=True, max_length=255, verbose_name='Title')),
                ('title_uk', models.CharField(blank=True, max_length=255, null=True, verbose_name='Title')),
            ],
            options={
                'verbose_name': 'Product flag',
                'verbose_name_plural': 'Product flags',
            },
        ),
    ]




from django.conf import settings
from django.test import TestCase

from mpshop.products.test_helpers import (
    create_category, create_categories, create_products)


class ProductTest(TestCase):

    def setUp(self):
        self.language_code = settings.LANGUAGE_CODE

    def test_list_contain_product_titles(self):

        category = create_category(name='Test category')

        products = create_products(category)

        url = '/%s/products/category/test_category_1/' % self.language_code

        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)

        for product in products:
            self.assertContains(response, product.title)

    def test_list_has_next_page_url(self):

        category = create_category(name='Test category')

        create_products(category, count=18)

        url = '/%s/products/category/test_category_1/' % (
            self.language_code)

        response = self.client.get(url)

        self.assertContains(response, url + '?page=2')

    def test_list_has_previous_page_url(self):

        category = create_category(name='Test category')

        create_products(category, count=18)

        url = '/%s/products/category/test_category_1/' % (
            self.language_code)

        response = self.client.get(url + '?page=2')

        self.assertContains(response, url + '?page=1')


class ProductCategoryTest(TestCase):

    def setUp(self):
        self.language_code = settings.LANGUAGE_CODE

    def test_category_slug(self):

        category = create_category(name='Test category')

        self.assertEqual(category.slug, 'test_category')

    def test_category_full_slug(self):

        parent = create_category(name='Parent')

        category = create_category(name='Child', parent=parent)

        self.assertEqual(category.full_slug, 'parent/child')

    def test_category_full_name(self):

        parent = create_category(name='Parent')

        category = create_category(name='Child', parent=parent)

        self.assertEqual(category.full_name, 'Parent > Child')

    def test_category_absolute_url(self):

        parent = create_category(name='Parent')

        category = create_category(name='Child', parent=parent)

        self.assertEqual(category.get_absolute_url(),
                         '/uk/products/category/parent/child_2/')

    def test_list_contain_category_names(self):

        categories = create_categories(count=3)

        response = self.client.get('/%s/products/' % self.language_code)

        self.assertEqual(response.status_code, 200)

        for category in categories:
            self.assertContains(response, category.name)




from django.test import TestCase
from django.test.client import RequestFactory
from django.template import Template, RequestContext

from shop.products.test_helpers import create_category, create_products

from shop.flags.models import ProductFlag


class FlagsTest(TestCase):

    def setUp(self):

        self.flags = create_flags
        self.products = create_products(category=create_category(), count=2)

    def test_set_currency_endpoint_changes_session_currency(self):

        url = '/currencies/set-currency/'

        response = self.client.post(url, {'currency': currency})

        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            int(self.client.session[CURRENCY_SESSION_KEY]), currency)

    # def test_get_currency_form_template_tag_returns_currency_form(self):
    #
    #     factory = RequestFactory()
    #
    #     request = factory.get('/')
    #
    #     request.session = {CURRENCY_SESSION_KEY: CURRENCY_UAH}
    #
    #     html = """
    #         {% load currencies %}
    #         {% get_currency_form as form %}
    #         {{ form }}
    #     """
    #
    #     response = Template(html).render(RequestContext(request))
    #
    #     self.assertIn('value="1" selected="selected">UAH', response)
    #     self.assertIn('value="2">USD', response)
    #     self.assertIn('value="3">EUR', response)




from django.apps import apps
from django.core.mail import mail_managers
from django.utils.translation import ugettext_lazy as _
from django.template.loader import render_to_string
from django.contrib.sites.models import Site


def send_new_price_offer_notification(offer):

    subject = _('New product price offer #%s') % offer.id

    current_site = Site.objects.get_current()

    context = {'offer': offer, 'site': current_site}

    html = render_to_string('offers/notifications/email.html', context)

    mail_managers(subject=subject, message='', html_message=html)

    if apps.is_installed('turbosms'):

        from turbosms.lib import send_sms_from_template

        send_sms_from_template('offers/notifications/sms.txt', context)




from copy import deepcopy

from django import forms
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ObjectDoesNotExist

from slugify import slugify_url

from modeltranslation.admin import TranslationTabularInline

from shop.attributes.constants import ATTR_TYPE_SELECT
from shop.attributes.models import (
    ProductAttribute, AttributeOption, VALUE_FIELDS)


class ProductAttributeForm(forms.ModelForm):

    def clean_option_group(self):
        data = self.cleaned_data

        option_group = data.get('option_group')

        if data.get('type') == ATTR_TYPE_SELECT and not option_group:
            raise forms.ValidationError(
                _('Option group is required for field type select'))

        return option_group

    def clean_slug(self):
        slug = self.cleaned_data.get('slug')

        if slug:
            return slug

        name = self.cleaned_data.get('name_%s' % settings.LANGUAGE_CODE)
        return slugify_url(name, separator='_')

    class Meta:
        model = ProductAttribute
        fields = '__all__'


class AttributeOptionInline(TranslationTabularInline):

    model = AttributeOption
    extra = 0


class ProductFormMixin(object):

    def __init__(self, *args, **kwargs):

        super(ProductFormMixin, self).__init__(*args, **kwargs)

        if self.instance.pk:
            self._build_attr_fields()

    def get_option_field_name(self, attr):
        return 'option_' + attr.full_slug

    def clean(self):
        data = super(ProductFormMixin, self).clean()

        for attr in self.instance.attr.all():

            if attr.has_options:
                new_option = data.get(self.get_option_field_name(attr))

                if new_option:
                    option, c = attr.option_group.options.get_or_create(
                        option=new_option)
                    data[attr.full_slug] = option

        return data

    def save(self, commit=True):
        product = super(ProductFormMixin, self).save(commit)

        if 'category' in self.changed_data:
            product.attribute_values.all().delete()

        return product

    def _build_attr_fields(self):

        fields = self.fields = deepcopy(self.base_fields)

        for attr in self.instance.attr.all():

            fields[attr.full_slug] = self._build_attr_field(attr)

            if attr.has_options:
                label = attr.name + unicode(_(' [New value]'))
                fields[self.get_option_field_name(attr)] = forms.CharField(
                    label=label, required=False)

            try:
                value = self.instance.attribute_values.get(
                    attribute=attr).value
            except ObjectDoesNotExist:
                pass
            else:
                self.initial[attr.full_slug] = value

    def _post_clean(self):

        for attr in self.instance.attr.all():

            if attr.full_slug in self.cleaned_data:
                value = self.cleaned_data[attr.full_slug]
                setattr(self.instance.attr, attr.slug, value)

        super(ProductFormMixin, self)._post_clean()

    def _build_attr_field(self, attr):

        kwargs = {'label': attr.name, 'required': attr.required}

        if attr.type is ATTR_TYPE_SELECT:
            kwargs['queryset'] = attr.option_group.options.all()
            return forms.ModelChoiceField(**kwargs)

        return VALUE_FIELDS[attr.type].formfield(**kwargs)



from django.apps import apps
from django.shortcuts import get_object_or_404, render
from django.views.generic.detail import DetailView

from pure_pagination import Paginator

from shop.products.forms import SearchProductForm


def product_list(request, category_slug, category_pk,
                 search_form_class=SearchProductForm):

    Product = apps.get_model('products', 'Product')
    ProductCategory = apps.get_model('products', 'ProductCategory')

    category = get_object_or_404(ProductCategory, pk=category_pk)

    categories = category.get_descendants(include_self=True)

    products = Product.visible.filter(category__in=categories)

    form = search_form_class(
        data=request.GET, products=products, category=category)

    paginator = Paginator(form.get_objects(), per_page=12, request=request)

    context = {
        'search_form': form,
        'category': category,
        'products': paginator.page(request.GET.get('page', 1))
    }

    return render(request, 'products/product_list.html', context)


def product_search(request):

    Product = apps.get_model('products', 'Product')

    form = SearchProductForm(Product.visible.all(), data=request.GET)

    paginator = Paginator(form.get_objects(), per_page=12, request=request)

    context = {
        'search_form': form,
        'products': paginator.page(request.GET.get('page', 1))
    }

    return render(request, 'products/search.html', context)


class ProductInfoView(DetailView):

    model = apps.get_model('products', 'Product')

    pk_url_kwarg = 'product_pk'

    context_object_name = 'product'

    template_name = 'products/info.html'

    def get_queryset(self):
        return self.model.visible.all()

    def update_viewed_products(self, count=6):

        request = self.request

        product_pk = self.kwargs.get(self.pk_url_kwarg)

        product_pks = request.session.get('viewed_product_pks', [])

        if product_pk in product_pks:
            product_pks.remove(product_pk)

        product_pks.insert(0, product_pk)

        if len(product_pks) > count:
            product_pks = product_pks[:count]

        request.session['viewed_product_pks'] = product_pks

        return product_pks

    def get_viewed_products(self):
        viewed_product_pks = self.update_viewed_products()
        return self.get_queryset().filter(pk__in=viewed_product_pks)

    def get_context_data(self, **kwargs):
        context = super(ProductInfoView, self).get_context_data(**kwargs)
        context['recently_viewed_products'] = self.get_viewed_products()
        return context




from shop.lib import format_price
from shop.currencies.settings import CURRENCIES, DEFAULT_CURRENCY


def format_printable_price(price, currency=DEFAULT_CURRENCY):
    return '%s %s' % (format_price(price), dict(CURRENCIES)[currency])




from django.conf.urls import url

from shop.cart import views


urlpatterns = [

    url(r'^$', views.index, name='index'),

    url(r'^add/$', views.add, name='add'),

    url(r'^remove/$', views.remove, name='remove'),

    url(r'^set-qty/$', views.set_qty, name='set-qty'),

]




from django.apps import apps
from django.shortcuts import render


def get_categories(request):

    root = request.GET.get('root', False)

    category_model = apps.get_model('products', 'ProductCategory')

    if root:
        categories = category_model.objects.root_nodes()
    else:
        categories = category_model.objects.all()

    context = {
        'categories': categories
    }

    return render(request, 'products/api/categories.html', context)




from django.apps import apps


CURRENCY_UAH = 1


def create_category(name='Category name', parent=None):
    ProductCategory = apps.get_model('products', 'ProductCategory')
    category = ProductCategory(name=name)

    if parent is not None:
        category.parent = parent

    category.save()

    return category


def create_categories(count=3):

    categories = []

    for i in xrange(1, count + 1):
        categories.append(create_category('Category name %s' % i))

    return categories


def create_product(
        category, title='Product title', price_in_currency=0,
        currency=CURRENCY_UAH):

    Product = apps.get_model('products', 'Product')

    product = Product(
        category=category, title=title, price_in_currency=price_in_currency,
        currency=currency)

    product.save()

    return product


def create_products(category, count=3):

    products = []

    for i in xrange(1, count + 1):
        products.append(create_product(
            category=category, title='Product title %s' % i))

    return products




from django import template

from shop.cart.lib import Cart


register = template.Library()


@register.assignment_tag(takes_context=True)
def get_cart(context, session_key=None, cart_class=Cart):
    return cart_class(context['request'].session, session_key=session_key)




from django.utils.translation import ugettext_lazy as _


def refresh_product_logos(modeladmin, request, queryset):
    for product in queryset:
        product.save()

refresh_product_logos.short_description = _('Refresh product logos')




from django.apps import apps
from django.shortcuts import render


def product_statistic(request):

    request.current_app = 'products'

    Product = apps.get_model('products', 'Product')
    ProductCategory = apps.get_model('products', 'ProductCategory')

    context = {
        'category_count': ProductCategory.objects.count(),
        'product_count': Product.objects.count()
    }

    return render(request, 'products/admin/statistic.html', context)




from django.test import TestCase
from django.test.utils import override_settings
from django.test.client import RequestFactory
from django.template import Template, RequestContext

from shop.currencies.models import ExchangeRate


CURRENCY_UAH = 1
CURRENCY_USD = 2
CURRENCY_EUR = 3


CURRENCIES = (
    (CURRENCY_UAH, 'UAH'),
    (CURRENCY_USD, 'USD'),
    (CURRENCY_EUR, 'EUR'),
)

USD_COURSE = 27.5
EUR_COURSE = 29

CURRENCY_SESSION_KEY = 'CURRENCY'


@override_settings(CURRENCIES=CURRENCIES, DEFAULT_CURRENCY=CURRENCY_UAH)
class ExchangeRateTest(TestCase):

    def setUp(self):
        ExchangeRate.objects.create(currency=CURRENCY_USD, value=USD_COURSE)
        ExchangeRate.objects.create(currency=CURRENCY_EUR, value=EUR_COURSE)

    def test_get_exchange_rates_returns_correct_data(self):

        exchange_rates = ExchangeRate.get_exchange_rates()

        self.assertEqual(exchange_rates[CURRENCY_UAH], 1)
        self.assertEqual(exchange_rates[CURRENCY_USD], 27.5)
        self.assertEqual(exchange_rates[CURRENCY_EUR], 29.0)

    def test_convert_from_uah_to_usd(self):

        self.assertEqual(
            ExchangeRate.convert(2750, CURRENCY_UAH, CURRENCY_USD), 100.0)

        self.assertEqual(
            ExchangeRate.convert(150, CURRENCY_UAH, CURRENCY_USD),
            5.454545454545454)

    def test_convert_from_uah_to_eur(self):

        self.assertEqual(
            ExchangeRate.convert(2900, CURRENCY_UAH, CURRENCY_EUR), 100.0)

        self.assertEqual(
            ExchangeRate.convert(100, CURRENCY_UAH, CURRENCY_EUR),
            3.4482758620689653)

    def test_convert_from_eur_to_uah(self):

        self.assertEqual(
            ExchangeRate.convert(100, CURRENCY_EUR, CURRENCY_UAH), 2900)

        self.assertEqual(
            ExchangeRate.convert(125, CURRENCY_EUR, CURRENCY_UAH), 3625.0)

    def test_convert_from_eur_to_usd(self):

        self.assertEqual(
            ExchangeRate.convert(100, CURRENCY_EUR, CURRENCY_USD),
            105.45454545454545)

        self.assertEqual(
            ExchangeRate.convert(125, CURRENCY_EUR, CURRENCY_USD),
            131.81818181818184)

    def test_convert_from_usd_to_uah(self):

        self.assertEqual(
            ExchangeRate.convert(100, CURRENCY_USD, CURRENCY_UAH), 2750)

        self.assertEqual(
            ExchangeRate.convert(125, CURRENCY_USD, CURRENCY_UAH), 3437.5)

    def test_convert_from_usd_to_eur(self):

        self.assertEqual(
            ExchangeRate.convert(100, CURRENCY_USD, CURRENCY_EUR),
            94.82758620689654)

        self.assertEqual(
            ExchangeRate.convert(125, CURRENCY_USD, CURRENCY_EUR),
            118.5344827586207)

    def test_convert_to_self_currency(self):

        for currency, name in CURRENCIES:
            self.assertEqual(
                ExchangeRate.convert(100, currency, currency), 100)

    def test_convert_with_printable_param_returns_printable_price(self):

        for currency, name in CURRENCIES:
            price = ExchangeRate.convert(
                100, currency, currency, printable=True)
            self.assertEqual(price, '100.00 %s' % name)

    def test_convert_with_format_price_param_returns_formatted_price(self):

        for currency, name in CURRENCIES:
            price = ExchangeRate.convert(
                100, currency, currency, format_price=True)
            self.assertEqual(price, '100.00')

    def test_convert_to_unknown_currency_raises_value_error(self):

        for currency, name in CURRENCIES:
            self.assertRaises(
                ValueError, ExchangeRate.convert, 100, currency, 5)

    def test_convert_from_unknown_currency_raises_value_error(self):

        for currency, name in CURRENCIES:
            self.assertRaises(
                ValueError, ExchangeRate.convert, 100, 5, currency)

    def test_set_currency_endpoint_changes_session_currency(self):

        for currency, name in CURRENCIES:

            url = '/currencies/set-currency/'

            response = self.client.post(url, {'currency': currency})

            self.assertEqual(response.status_code, 302)

            self.assertEqual(
                int(self.client.session[CURRENCY_SESSION_KEY]), currency)

    def test_get_currency_form_template_tag_returns_currency_form(self):

        factory = RequestFactory()

        request = factory.get('/')

        request.session = {CURRENCY_SESSION_KEY: CURRENCY_UAH}

        html = """
            {% load currencies %}
            {% get_currency_form as form %}
            {{ form }}
        """

        response = Template(html).render(RequestContext(request))

        self.assertIn('value="1" selected="selected">UAH', response)
        self.assertIn('value="2">USD', response)
        self.assertIn('value="3">EUR', response)
